----------------------------------------------------------------------------------
--
-- SetupMetricsConnection
--
-- Set ups the connection to Metrics. This script is responsible for authenticating to Metrics server.
--
--
-- Configuration information is maintained in temporary tables described below.
-- Set appropriate values to the following lines of code or comment lines out if
-- the values have been set before calling this script (e.g. #SubScriptId has been set).
--
-- #BaseUrl 			URL of the MEA web service interface, e.g. 'http://localhost/QPR2017-1/Portal/QPR.Isapi.dll/wsforward/MainService.svc/webHttp'
-- #LogOnName 			user name for MEA login
-- #Password			password for MEA login
-- #MetricsModelName 	name of the Metrics model where data is to be loaded
----------------------------------------------------------------------------------
-- SELECT 'http://localhost/QPR2017-1/Portal/QPR.Isapi.dll/wsforward/MainService.svc/webHttp' AS VAL INTO #BaseUrl;
-- SELECT 'qpr' AS VAL INTO #LogOnName;
-- SELECT 'demo' AS VAL INTO #Password;
-- SELECT 'Metrics automation' AS VAL INTO #MetricsModelName

-- Check that all the necessary temporary tables exist!
(SELECT 'MessageText', 'Temporary table #BaseUrl must be set before calling SetupMetricsConnection.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#BaseUrl') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', 'Temporary table #LogOnName must be set before calling SetupMetricsConnection.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#LogOnName') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', 'Temporary table #Password must be set before calling SetupMetricsConnection.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#Password') IS NOT NULL,'0','1'))
--#Exit 

(SELECT 'MessageText', 'Temporary table #MetricsModelName must be set before calling SetupMetricsConnection.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#MetricsModelName') IS NOT NULL,'0','1'))
--#Exit 

---------------------
----AUTHENTICATING
---------------------

DECLARE @baseurl NVARCHAR(MAX) = (SELECT VAL FROM [#BaseUrl]);
DECLARE @logonname NVARCHAR(MAX) = (SELECT VAL FROM [#LogOnName]);
DECLARE @password NVARCHAR(MAX) = (SELECT VAL FROM [#Password]);

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', @baseurl + '/Authenticate') UNION ALL
(SELECT 'Body', '{"logOnName":"' + @logonname + '","password":"' + @password + '"}') UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*')
--#CallWebService

print 'Response succcess: ' + CONVERT(VARCHAR(1),@_ResponseSuccess)
print 'Authenticate result: [' + @_ResponseText + ']'

-- Have to save the response text to a temporary table
-- to survive over the --Exit line.
IF OBJECT_ID('tempdb..#ResponseText') IS NOT NULL
	DROP TABLE #ResponseText;
SELECT @_ResponseText AS VAL INTO #ResponseText;


-- If CallWebService was successful continue, else exit
(SELECT 'MessageText', 'Authentication to Metrics failed!') UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,@_ResponseSuccess) = 1,'0','1'))
--#Exit 

---------------------------------------------------------------------------
---- STORE Session ID into #SessionData
---------------------------------------------------------------------------
DECLARE @sessionId NVARCHAR(MAX);
DECLARE @pos INT;
DECLARE @ResponseText NVARCHAR(MAX) = (SELECT VAL FROM #ResponseText)

-- Extract session id
SET @pos = PATINDEX('%"d":"%',@ResponseText);
IF @pos > 0
BEGIN
  SET @sessionId = SUBSTRING(@ResponseText, @pos + 5, LEN(@ResponseText))
  SET @pos = PATINDEX('%"%',@sessionId);
  IF @pos > 0
  BEGIN
    SET @sessionId = SUBSTRING(@sessionId, 0, @pos)
    PRINT 'Session id: ' + @sessionId;
  END;
END;

IF OBJECT_ID('tempdb..#SessionData') IS NOT NULL
	DROP TABLE #SessionData;
SELECT @sessionId AS SES_ID INTO #SessionData;

GO;

---------------------------------------------------------------------------
---- Find model ID according to the model name defined in #MetricsModelName
---------------------------------------------------------------------------

DECLARE @ModelName AS NVARCHAR(MAX);
DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);

SELECT @ModelName = [VAL] FROM #MetricsModelName
SELECT @SessionString = [SES_ID] FROM #SessionData;
SELECT @Body = '{"sessionId":"' + @SessionString + '",'
+ '"query":"[SC].Models(criteria=\"Find(\\\"' + @ModelName + '\\\",Name)\")",'
+ ' "criteria":"", "sortBy":"", "attributes":"name" ,"options":""}';

print @Body

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/QueryObjectsAsXML') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*')
--#CallWebService

DELETE FROM #ResponseText;
INSERT INTO #ResponseText SELECT @_ResponseText;

-- If CallWebService was successful continue, else exit
(SELECT 'MessageText', 'QueryObjectsAsXML failed!') UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,@_ResponseSuccess) = 1,'0','1'))
--#Exit 

DECLARE @pos INT;
DECLARE @ResponseText NVARCHAR(MAX) = (SELECT VAL FROM #ResponseText);
DECLARE @xmlString NVARCHAR(MAX);
DECLARE @x xml;

-- Extract ResultSet XML
SET @pos = PATINDEX('%"d":"%',@ResponseText);
IF @pos > 0
BEGIN
  SET @xmlString = REPLACE(SUBSTRING(@ResponseText, @pos + 5, (LEN(@ResponseText)-8)), '\"', '"');
  SET @xmlString = REPLACE(@xmlString, '\u000d', '');
  SET @xmlString = REPLACE(@xmlString, '\u000a', '');
  SET @xmlString = REPLACE(@xmlString, '\/', '/');
  
--  PRINT 'XML ResultSet Len: ' + CONVERT(VARCHAR, LEN(@ResponseText))
--  PRINT 'XML ResultSet: ' + @xmlString;
END;

SET @x = @xmlString

DECLARE @results AS NVARCHAR(MAX);
DECLARE @modelId AS NVARCHAR(MAX);

SELECT @results = @x.value('(/ResultSetHierarchy/@totalResults)[1]', 'int');
SELECT @modelId = @x.value('(/ResultSetHierarchy/object/@id)[1]', 'varchar(100)');
 
print 'Results: ' + CONVERT(VARCHAR,@results);
print 'ModelId: ' + CONVERT(VARCHAR,@modelId);

IF OBJECT_ID('tempdb..#ModelId') IS NOT NULL
	DROP TABLE #ModelId;
SELECT @ModelId AS VAL INTO #ModelId
